/*
   FNT - SQL-backend for SQLite3 - function query

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <sqlite3.h>
#include <stdlib.h>
#include <string.h>

#include "../include/global.h"

#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/sql.h"

#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT fnt_sql_query_t *
query (fnt_sql_connect_t * sql_connect, const char *query_string)
{
    fnt_sql_query_t *ret_value;
    sqlite3_stmt *result, *result2;
    char *count_query;

    if ((sql_connect == NULL) || (query_string == NULL))
	return NULL;

    ret_value = g_try_malloc (sizeof (fnt_sql_query_t));
    if (ret_value == NULL)
	return NULL;

    /* sqlite3_complete(const char *sql); (http://www.sqlite.org/c3ref/complete.html) */

    if (sqlite3_prepare_v2
	((sqlite3 *) sql_connect->db_handle, query_string, -1, &result,
	 NULL) != SQLITE_OK)
    {
	fnt_error_push (FNT_MODULE_sql, FNT_SQL_E_DRIVER,
			"%s",
			sqlite3_errmsg ((sqlite3 *) sql_connect->db_handle));
	sqlite3_finalize (result);
	g_free (ret_value);
	return NULL;
    }
    sqlite3_step (result);
    ret_value->num_fields = sqlite3_column_count (result);

    ret_value->is_result_present = (ret_value->num_fields) ? true : false;

    if (ret_value->is_result_present)
    {
	count_query =
	    g_strdup_printf ("SELECT COUNT(*) FROM (%s)", query_string);
	sqlite3_prepare_v2 ((sqlite3 *) sql_connect->db_handle, count_query,
			    -1, &result2, NULL);
	g_free (count_query);

	sqlite3_step (result2);

	count_query = sqlite3_column_text (result2, 0);

	if (count_query != NULL)
	    ret_value->num_rows = atoll (count_query);
	else
	    ret_value->num_rows = 0;

	sqlite3_finalize (result2);
    }
    else
    {
	ret_value->num_rows = sqlite3_changes (sql_connect->db_handle);
    }

    ret_value->__current_row = -1;
    ret_value->handle = (void *) result;
    return ret_value;
}
